﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SE.GOV.MM.Integration.Log
{

    /// <summary>
    /// Handles Log messages and where they appear
    /// </summary>
    public static class LogManager
    {

        /// <summary>
        /// key in app.config or web.config "WriteToFile"
        /// </summary>
        private static bool WriteToFile 
        { 
            get 
            { 
                var writeToFile = false;
                bool.TryParse(ConfigurationManager.AppSettings["WriteToFile"], out writeToFile);

                return writeToFile;
            } 
        }

        /// <summary>
        /// key in app.config or web.config "LogFilePath"
        /// </summary>
        private static string LogFilePath { get { return ConfigurationManager.AppSettings["LogFilePath"]; } }

        /// <summary>
        /// Create log in either eventlog, file or both. Depends on key in app.config or web.config appSettings "WriteToFile" and "LogFilePath".
        /// </summary>
        /// <param name="log"></param>
        public static void Log(Log log)
        {
            try
            {                           
                if (WriteToFile)
                {
                    var logMessage = createLogMessage(log);
                    LogFileWriter.WriteLogMessageToFile(LogFilePath, logMessage);
                }

                switch (log.Level)
                {
                    case Level.Error:
                        EventLogWriter.WriteExceptionToEventLog(log);
                        break;
                    case Level.Info:
                        EventLogWriter.WriteInfoToEventLog(log);
                        break;
                    case Level.Warning:
                        EventLogWriter.WriteWarningToEventLog(log);
                        break;
                    case Level.Trace:
                        //Dont write to eventlog
                        break;
                }

            }
            catch (Exception e)
            {

            }
        }

        /// <summary>
        /// Writes to file, key in app.config or web.config "WriteToFile"
        /// </summary>
        /// <param name="logMessage"></param>
        public static void LogTrace(string logMessage)
        {
            if (WriteToFile)
            {
                var messageLog = string.Format("{0} TRACE Message: {1}", DateTime.Now.ToLocalTime().ToString(), logMessage);
                LogFileWriter.WriteLogMessageToFile(LogFilePath, messageLog);
            }
        }

        
        /// <summary>
        /// Example: 2015-09-02 11:30:20.2327 TRACE SE.GOV.MM.Integration.Outlook.Service.MessageService: leaving SendMessage
        /// Converts a Log object to a string message.
        /// </summary>
        /// <param name="log"></param>
        /// <returns></returns>
        private static string createLogMessage(Log log)
        {
            var message = string.Empty;

            if (log.Exception != null)
            {
                message = string.Format("{0} {1} Message: {2} Exception: {3}", DateTime.Now.ToLocalTime().ToString(), log.Level, log.Message, log.Exception);
            }
            else
            {
                message = string.Format("{0} {1} Message: {2}", DateTime.Now.ToLocalTime().ToString(), log.Level, log.Message);
            }
            return message;
        }
    }
}
