﻿using SE.GOV.MM.Integration.Outlook.BusinessLayer.BusinessObject;
using SE.GOV.MM.Integration.Outlook.BusinessLayer.Helper;
using SE.GOV.MM.Integration.Package.DataTransferObjects.BusinessObjects;
using System;
using System.Collections.Generic;

namespace SE.GOV.MM.Integration.Outlook.Helper
{

    /// <summary>
    /// Creates a new Mail from an Outlook MailItem.
    /// </summary>
    public class MailHelper
    {

        /// <summary>
        /// Convert outlook Mailitem to object specified in Message service.
        /// </summary>
        public Mail CreateMailItem(string sender, string recipient, string subject, string bodyText, string reference)
        {         
            Mail mailItem = new Mail()
            {
                CorrealationId = string.Empty,
                MessageHeaderId = "1",
                Reference = reference,
                Sender = sender,
                Subject = subject
            };

            mailItem.Attachments = new List<Attachment>();

            mailItem.Recipient = new Recipient()
            {
                To = recipient
            };

            // Set body
            var body = new Body();
            body.Text = bodyText;
            body.ContentType = "text/plain";
            mailItem.Body = body;

            return mailItem;
        }

        public void Validate(string recipient, string subject)
        {
            var rHelper = new RecipientHelper();
            if (!rHelper.ValidRecipient(recipient))
            {
                throw new ArgumentException(ResourceHelper.Resources[3].Text);
            }

            if (string.IsNullOrEmpty(subject))
            {
                throw new ArgumentException(ResourceHelper.Resources[5].Text);
            }

        }

        public MailBox GetMailBoxFromIsReachableResult(IsReachableResult isReachableResult)
        {
            return new MailBox()
            {
                Id = isReachableResult.Id,
                Name = isReachableResult.Name,
                ServiceAdress = isReachableResult.ServiceAdress,
                UIAdress = isReachableResult.UIAdress
            };
        }
    }
}
